//
// CYBERSOURCE CORPORATION ("COMPANY") CONFIDENTIAL
// Copyright (c) 2021 CyberSource Corporation. All Rights Reserved.
//
// NOTICE:  All information contained herein is, and remains the property of COMPANY. The intellectual and technical concepts contained
// herein are proprietary to COMPANY and may be covered by European or foreign Patents, patents in process, and are protected by trade secret or copyright law.
// Dissemination of this information or reproduction of this material is strictly forbidden unless prior written permission is obtained
// from COMPANY.  Access to the source code contained herein is hereby forbidden to anyone except current COMPANY employees, managers or contractors who have executed
// Confidentiality and Non-disclosure agreements explicitly covering such access.
//
// The copyright notice above does not evidence any actual or intended publication or disclosure of this source code, which includes
// information that is confidential and/or proprietary, and is a trade secret, of COMPANY.
// ANY REPRODUCTION, MODIFICATION, DISTRIBUTION, PUBLIC  PERFORMANCE,
// OR PUBLIC DISPLAY OF OR THROUGH USE  OF THIS  SOURCE CODE  WITHOUT  THE EXPRESS WRITTEN CONSENT OF COMPANY IS STRICTLY PROHIBITED, AND IN VIOLATION OF APPLICABLE
// LAWS AND INTERNATIONAL TREATIES.  THE RECEIPT OR POSSESSION OF  THIS SOURCE CODE AND/OR RELATED INFORMATION DOES NOT CONVEY OR IMPLY ANY RIGHTS
// TO REPRODUCE, DISCLOSE OR DISTRIBUTE ITS CONTENTS, OR TO MANUFACTURE, USE, OR SELL ANYTHING THAT IT  MAY DESCRIBE, IN WHOLE OR IN PART.

#import "MPTransactionEnums.h"
@class MPPaymentDetails;
@class MPProcessingDetails;
@class MPStatusDetails;
@class MPRefundDetails;
@class MPClearingDetails;
@class MPTransactionDetails;
@class MPReceipt;
@class MPCardDetails;
@class MPShopperDetails;
@class MPGiftCardDetails;
@class MPTransactionVerificationResults;
@class MPDccDetails;


/**
 * A transaction contains all information required to make a transaction. It goes through specifically defined states and offers additional details after successful execution.
 * @since 2.0.0
 */
@interface MPTransaction : NSObject

- (nonnull instancetype)init UNAVAILABLE_ATTRIBUTE;
+ (nonnull instancetype)new UNAVAILABLE_ATTRIBUTE;

#pragma mark -
#pragma mark General Information
/**
 * @name General Information
 */

/**
 * The amount of the transaction.
 * @since 2.0.0
 */
@property (strong, readonly, nonatomic, nonnull) NSDecimalNumber *amount;

/**
 * The currency used for the transaction.
 * @since 2.0.0
 */
@property (assign, readonly, nonatomic) MPCurrency currency;

/**
 * The subject line for the transaction.
 * @since 2.0.0
 */
@property (strong, readonly, nonatomic, nullable) NSString *subject;

/**
 * An arbitrary string to be displayed on your customer's credit card statement.
 * @since 2.4.7
 */
@property (strong, readonly, nonatomic, nullable) NSString *statementDescriptor;

#pragma mark -
#pragma mark Transaction Setup (must be set manually)
/**
 * @name Transaction Setup (must be set manually)
 */

/**
 * The type of transaction to be executed.
 * @since 2.0.0
 */
@property (assign, readonly, nonatomic) MPTransactionType type;

/**
 * The mode the transaction was executed
 * @since 2.14.0
 */
@property (assign, readonly, nonatomic) MPTransactionMode mode;

/**
 * Indicates if the transaction has been captured.
 * @since 2.8.0
 */
@property (assign, readonly, nonatomic) BOOL captured;

#pragma mark -
#pragma mark Payment Details
/**
 * @name Payment Details
 */

/**
 * The payment details used for the transaction.
 * @since 2.0.0
 */
@property (strong, readonly, nonatomic, nonnull) MPPaymentDetails *paymentDetails;

/**
 * The refund details associated with the transaction.
 * @since 2.0.0
 */
@property (strong, readonly, nonatomic, nonnull) MPRefundDetails *refundDetails;

/**
 * The clearing details associated with the transaction.
 * @since 2.6.0
 */
@property (strong, readonly, nonatomic, nonnull) MPClearingDetails *clearingDetails;

/**
 * The details of the card used for this transaction
 * @since 2.8.0
 */
@property (strong, readonly, nonatomic, nonnull) MPCardDetails *cardDetails;

/**
 * The details of the shopper
 * @since 2.8.0
 */
@property (strong, readonly, nonatomic, nonnull) MPShopperDetails *shopperDetails;

/**
 * Gift card details, e.g. balance or expiry date.
 * @since 2.32.0
 */
@property (strong, readonly, nonatomic, nonnull) MPGiftCardDetails *giftCardDetails;

/**
 * Extra transaction details, e.g. fees or tips.
 * @since 2.7.0
 */
@property (strong, readonly, nonatomic, nonnull) MPTransactionDetails *details;

#pragma mark -
#pragma mark Transaction Result

/**
 * Current state of the transaction. Approved, Declined and Error are the final states.
 * @since 2.0.0
 */
@property (assign, readonly, nonatomic) MPTransactionStatus status;

/**
 * Additional details associated with the current status, providing more detailed information for the current status and debug information.
 * @since 2.0.0
 */
@property (strong, readonly, nonatomic, nullable) MPStatusDetails *statusDetails;

/**
 * The current workflow state the transaction is in.
 * @since 2.0.0
 */
@property (assign, readonly, nonatomic) MPTransactionState state;

/**
 * If the transaction execution fails for any reason, this will contain the last error message. Otherwise nil.
 * @since 2.0.0
 */
@property (strong, readonly, nonatomic, nullable) NSError *error;

#pragma mark -
#pragma mark Global References
/**
 * @name Global Refrences
 */

/**
 * The (temporary) session that references the transaction.
 * @since 2.0.0
 */
@property (strong, readonly, nonatomic, nullable) NSString *sessionIdentifier;

/**
 * The reference within our infrastructure to identify the transaction (payworks identifier).
 * @since 2.0.0
 */
@property (strong, readonly, nonatomic, nullable) NSString *identifier;

/**
 * A reference to a group of transactions that logically belong together.
 * @since 2.0.0
 */
@property (strong, readonly, nonatomic, nullable) NSString *groupIdentifier;

/**
 * A reference to a transaction that the current transaction is building up on.
 * @since 2.0.0
 */

@property (strong, readonly, nonatomic, nullable) NSString *referencedTransactionIdentifier;

/**
 * An identifier that can be set by the developer/user. Must be in the range [0-9a-zA-Z]. This will normally be a reference to a order in your system.
 * @since 2.0.0
 */
@property (strong, readonly, nonatomic, nullable) NSString *customIdentifier;

/**
 * Timestamp when the transaction was created. This timestamp will always be UTC.
 * @since 2.0.0
 */
@property (strong, readonly, nonatomic, nullable) NSDate *created;

/**
 * The timezone offset in which the transaction was performed. When combining this information with `created` (always UTC) you are able to calculate the local transaction time.
 * @since 2.31.0
 */
@property (strong, readonly, nonatomic, nullable) NSTimeZone *createdTimeZone;

/**
 * Indicates if the transaction can still be aborted.
 * @return YES if the transaction can be aborted, NO otherwise
 * @since 2.0.0
 */
- (BOOL)canBeAborted;

#pragma mark -
#pragma mark Customer Information

/**
 * Information and identifiers returned by the processor.
 * @since 2.0.0
 */
@property (strong, readonly, nonatomic, nonnull) MPProcessingDetails *processingDetails;

#pragma mark -
#pragma mark Customer Information

/**
 * Receipt that can be printed for the customer
 * @since 2.6.0
 */
@property (strong, readonly, nonatomic, nullable) MPReceipt *customerReceipt;

/**
 * Receipt that can be printed for the merchant
 * @since 2.6.0
 */
@property (strong, readonly, nonatomic, nullable) MPReceipt *merchantReceipt;

/**
 * Determines transaction expected workflow. By default is MPTransactionWorkflowTypeUnknown.
 * @since 2.27.0
 */
@property (readonly, nonatomic) MPTransactionWorkflowType workflow;

/**
 * Verification result for MOTO transaction.
 * @since 2.27.0
 */
@property (readonly, nonatomic, nonnull) MPTransactionVerificationResults *verificationResults;

/**
 * The details of the DCC (Dynamic Currency Conversion)
 * @since 2.29.0
 */
@property (readonly, nonatomic, nonnull) MPDccDetails *dccDetails;

@end
