//
//  MPTransactionEnums.h
//  mpos_core
//
//  Created by Guri, Aleix on 16/05/2022.
//

NS_ASSUME_NONNULL_BEGIN
/**
 * Actions requested during a transaction. Bitwise OR to stack, AND to split.
 * @since 2.0.0
 */
typedef NS_OPTIONS(NSUInteger, MPTransactionAction) {
    /** No actions supported, NOP action */
    MPTransactionActionNone = 0,
    /** A customer signature must be provided to complete the transaction */
    MPTransactionActionCustomerSignature = 1 << 0,
    /** The customer must be identified by means of ID check with an official document */
    MPTransactionActionCustomerIdentification = 1 << 1,
    /** Used to indicate that an application needs to be selected in order to continue with the transaction */
    MPTransactionActionApplicationSelection = 1 << 2,
    /** Used to indicate that a credit or debit workflow needs to be selected */
    MPTransactionActionCreditDebitSelection = 1 << 3,
};
static MPTransactionAction MPTransactionActionAll =
        MPTransactionActionCustomerSignature | MPTransactionActionCustomerIdentification | MPTransactionActionApplicationSelection | MPTransactionActionCreditDebitSelection;

/**
 * The type of transaction that will be executed.
 * @since 2.0.0
 */
typedef NS_ENUM(NSUInteger, MPTransactionType) {
    /** Unknown or not available */
    MPTransactionTypeUnknown = 0,
    /** Debit transaction, where the funds are directly removed from debited from the account */
    MPTransactionTypeCharge,
    /** Refund transaction, where the funds are directly posted to the account */
    MPTransactionTypeRefund,
    /** Preauthorize transaction, where the funds are reserved on the account. Note: just available for mapping purposes and cannot be initiated by the SDK.  */
    MPTransactionTypePreauthorize,
    /** Credit transaction, where the funds are credited to an account. Note: just available for mapping purposes and cannot be initiated by the SDK. */
    MPTransactionTypeCredit,
    /** Account verification transaction, where the card is verified as valid, no funds are taken from the account. */
    MPTransactionTypeVerification,
    /** Activation transaction, where the funds are transferred to a gift card account. **/
    MPTransactionTypeActivation
};

/**
 * The final result of the transaction.
 * @since 2.0.0
 */
typedef NS_ENUM(NSUInteger, MPTransactionStatus) {
    /** Unknown or not available */
    MPTransactionStatusUnknown = 0,
    /** Transaction is initialized and can be started */
    MPTransactionStatusInitialized,
    /** Transaction result is still pending (e.g. not finished or waiting for async workflow) */
    MPTransactionStatusPending,
    /** Transaction was accepted offline and must be uploaded to be executed by the backend */
    MPTransactionStatusAccepted,
    /** Transaction was approved */
    MPTransactionStatusApproved,
    /** Transaction was declined */
    MPTransactionStatusDeclined,
    /** Transaction was aborted (by merchant or shopper) */
    MPTransactionStatusAborted,
    /** An error occured, see [MPTransaction.error] for more details */
    MPTransactionStatusError,
    /** The transaction ended in a state that is inconclusive and the SDK cannot derive the outcome (e.g. due to no internet connection). This is a special case of failure. */
    MPTransactionStatusInconclusive
};

/**
 * The currency used for a transaction.
 * @since 2.0.0
 */
typedef NS_ENUM(NSUInteger, MPCurrency) {
    /** An unknown currency. Is actually part of the ISO standard, but should be treated as an error. */
    MPCurrencyUnknown,
    /** United Arab Emirates Dirham */
    MPCurrencyAED,
    /** Afghan Afghani */
    MPCurrencyAFN,
    /** Albanian Lek */
    MPCurrencyALL,
    /** Armenian Dram */
    MPCurrencyAMD,
    /** Argentine Peso */
    MPCurrencyARS,
    /** Australian Dollar */
    MPCurrencyAUD,
    /** Azerbaijani Manat */
    MPCurrencyAZN,
    /** Barbadian Dollar */
    MPCurrencyBBD,
    /** Bangladeshi Taka */
    MPCurrencyBDT,
    /** Bulgarian Lev */
    MPCurrencyBGN,
    /** Bahraini Dinar */
    MPCurrencyBHD,
    /** Bermudan Dollar */
    MPCurrencyBMD,
    /** Brunei Dollar */
    MPCurrencyBND,
    /** Bolivian Boliviano */
    MPCurrencyBOB,
    /** Brazilian Real */
    MPCurrencyBRL,
    /** Bahamian Dollar */
    MPCurrencyBSD,
    /** Botswana Pula */
    MPCurrencyBWP,
    /** Belize Dollar */
    MPCurrencyBZD,
    /** Canadian Dollar */
    MPCurrencyCAD,
    /** Swiss Franc */
    MPCurrencyCHF,
    /** Chilean Peso */
    MPCurrencyCLP,
    /** Chinese Yuan */
    MPCurrencyCNY,
    /** Colombian Peso */
    MPCurrencyCOP,
    /** Costa Rican Colon */
    MPCurrencyCRC,
    /** Czech Koruna */
    MPCurrencyCZK,
    /** Danish Krone */
    MPCurrencyDKK,
    /** Dominican Peso */
    MPCurrencyDOP,
    /** Algerian Dinar */
    MPCurrencyDZD,
    /** Egyptian Pound */
    MPCurrencyEGP,
    /** Euro */
    MPCurrencyEUR,
    /** Pound Sterling */
    MPCurrencyGBP,
    /** Ghanaian Cedi */
    MPCurrencyGHS,
    /** Guatemalan Quetzal */
    MPCurrencyGTQ,
    /** Hong Kong Dollar */
    MPCurrencyHKD,
    /** Honduran Lempira */
    MPCurrencyHNL,
    /** Croatian Kuna */
    MPCurrencyHRK,
    /** Hungarian Forint */
    MPCurrencyHUF,
    /** Indonesian Rupiah */
    MPCurrencyIDR,
    /** Israeli New Shekel */
    MPCurrencyILS,
    /** Indian Rupee */
    MPCurrencyINR,
    /** Jamaican Dollar */
    MPCurrencyJMD,
    /** Jordanian Dinar */
    MPCurrencyJOD,
    /** Japanese Yen */
    MPCurrencyJPY,
    /** Kenyan Shilling */
    MPCurrencyKES,
    /** South Korean Won */
    MPCurrencyKRW,
    /** Kuwaiti Dinar */
    MPCurrencyKWD,
    /** Kazakhstani Tenge */
    MPCurrencyKZT,
    /** Lebanese Pound */
    MPCurrencyLBP,
    /** Sri Lankan Rupee */
    MPCurrencyLKR,
    /** Liberian Dollar */
    MPCurrencyLRD,
    /** Moroccan Dirham */
    MPCurrencyMAD,
    /** Macanese Pataca */
    MPCurrencyMOP,
    /** Mauritian Rupee */
    MPCurrencyMUR,
    /** Mexican Peso */
    MPCurrencyMXN,
    /** Malaysian Ringgit */
    MPCurrencyMYR,
    /** Nigerian Naira */
    MPCurrencyNGN,
    /** Norwegian Krone */
    MPCurrencyNOK,
    /** New Zealand Dollar */
    MPCurrencyNZD,
    /** Omani Rial */
    MPCurrencyOMR,
    /** Panamanian Balboa */
    MPCurrencyPAB,
    /** Peruvian Nuevo Sol */
    MPCurrencyPEN,
    /** Philippine Peso */
    MPCurrencyPHP,
    /** Pakistani Rupee */
    MPCurrencyPKR,
    /** Polish Zloty */
    MPCurrencyPLN,
    /** Qatari Rial */
    MPCurrencyQAR,
    /** Romanian Leu */
    MPCurrencyRON,
    /** Serbian Dinar */
    MPCurrencyRSD,
    /** Russian Ruble */
    MPCurrencyRUB,
    /** Saudi Riyal */
    MPCurrencySAR,
    /** Seychellois Rupee */
    MPCurrencySCR,
    /** Swedish Krona */
    MPCurrencySEK,
    /** Singapore Dollar */
    MPCurrencySGD,
    /** Thai Baht */
    MPCurrencyTHB,
    /** Tunisian Dinar */
    MPCurrencyTND,
    /** Turkish Lira */
    MPCurrencyTRY,
    /** Trinidad & Tobago Dollar */
    MPCurrencyTTD,
    /** New Taiwan Dollar */
    MPCurrencyTWD,
    /** Ukrainian Hryvnia */
    MPCurrencyUAH,
    /** US Dollar */
    MPCurrencyUSD,
    /** Vietnamese Dong */
    MPCurrencyVND,
    /** East Caribbean Dollar */
    MPCurrencyXCD,
    /** Yemeni Rial */
    MPCurrencyYER,
    /** South African Rand */
    MPCurrencyZAR
};

/**
 * The state the transaction is currently in. Not all transactions run through all of the steps.
 * @since 2.0.0
 */
typedef NS_ENUM(NSUInteger, MPTransactionState) {
    /** Unkndown or not available */
    MPTransactionStateUnknown = 0,
    /** The transaction is waiting to be executed */
    MPTransactionStateIdle,
    /** The transaction is waiting for the card to be inserted */
    MPTransactionStateAwaitingCardPresentation,
    /** The card is currently being analyzed */
    MPTransactionStateAwaitingCardIdentification,
    /** The card requires an application selection by the cardholder */
    MPTransactionStateAwaitingApplicationSelection,
    /** The card requires credit/debit to be selected */
    MPTransactionStateAwaitingCreditDebitSelection,
    /** The transaction requires an amount confirmation by the cardholder */
    MPTransactionStateAwaitingAmountConfirmation,
    /** The transaction requires a PIN to be inserted */
    MPTransactionStateAwaitingPIN,
    /** The transaction is a being executed with the remote system */
    MPTransactionStateAwaitingExecution,
    /** The transaction is waiting for the card to be removed */
    MPTransactionStateAwaitingCardRemoval,
    /** The transaction requires a signature by the cardholder */
    MPTransactionStateAwaitingSignature,
    /** The transaction requires an identificatio by the cardholder */
    MPTransactionStateAwaitingIdentification,
    /** The transaction is being finalized with the remote host */
    MPTransactionStateAwaitingCompletion,
    /** The transaction is waiting for the card data input */
    MPTransactionStateAwaitingCardDataInput,
    /** The transaction is waiting for address input */
    MPTransactionStateAwaitingAddressInput,
    /** The transaction is waiting for verification data confirmation */
    MPTransactionStateAwaitingVerificationResultsConfirmation,
    /** The transaction is in a final state and no changes can be made to it anymore */
    MPTransactionStateEnded
};


/**
 * The mode the transaction was executed
 * @since 2.14.0
 */
typedef NS_ENUM(NSUInteger, MPTransactionMode) {
    /** Transaction was executed and finalized online*/
    MPTransactionModeOnline = 0,
    /** Transaction was executed offline*/
    MPTransactionModeOffline
};

/**
 * Describes the type of transaction workflow.
 * @since 2.27.0
 */
typedef NS_ENUM(NSUInteger, MPTransactionWorkflowType) {
    MPTransactionWorkflowTypeUnknown,
    MPTransactionWorkflowTypePos,
    MPTransactionWorkflowTypeAlternativePaymentMethod,
    MPTransactionWorkflowTypeMoto,
    MPTransactionWorkflowTypeGiftCard
};

NS_ASSUME_NONNULL_END
